/*
 * Project name:
     tRF click (Using mikroE's tRF click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20140416:
       - initial release (FJ);
 * Description:
      A simple example of using tRF click board, which will send the text string
      via the Telit's LE70-868 RF module when PORTD.B3 button is pressed and
      display the received string on the Lcd.
 * Test configuration:
     MCU:             P32MX795F512L
                      http://ww1.microchip.com/downloads/en/DeviceDoc/61156F.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    tRF click board - ac:tRF_click
                      2x16 lcd character display - ac:Lcd
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 * NOTES:
     - Place tRF click board in the mikroBUS socket 1 on the EasyPIC Fusion v7 board.
     - Pull-down PORTD.B3, Pull-up SW10.5 on the EasyPIC Fusion v7 board.
 */

// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATD7_bit;
sbit TFT_BLED at LATD2_bit;
sbit TFT_RS at LATD9_bit;
sbit TFT_CS at LATD10_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISD7_bit;
sbit TFT_BLED_Direction at TRISD2_bit;
sbit TFT_RS_Direction at TRISD9_bit;
sbit TFT_CS_Direction at TRISD10_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

sbit Send_Button at PORTB.B3;
sbit Send_Button_Direction at TRISB.B3;

char rxBuff[17];                         // Buffer string for storing data
char rxCnt = 0;                          // Counter for data writen in buffer string
unsigned short Received = 0;             // Received data flag
char lab1[17] = {0};

void updateLabel(char *newText, int x, int y, char* lbl){
  TFT_Set_Font(&TFT_defaultFont, CL_WHITE, FO_HORIZONTAL); // Color of background
  TFT_Write_Text(lbl, x, y);
  strcpy(lbl, newText);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL); // Color of text
  TFT_Write_Text(lbl, x, y);
  delay_ms(50);
}

void DrawFrame() {
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("tRF  Click  Board  Demo", 25, 14);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyPIC Fusion v7", 19, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLUE, FO_HORIZONTAL);
  TFT_Write_Text("Send string with RD3", 40, 140);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Recieved string:", 40, 80);
}

// Interrupt routine
void UART2interrupt() iv IVT_UART_2 ilevel 6 ics ICS_AUTO {
  char tmp;                              // Temporary variable declaration
    tmp = UART2_Read();                  // Store received data in the temporary variable
    if ((tmp == '\r') || (rxCnt > 15)) { // Check if data counter is larger than the buffer string size
                                         // or the received data is equal to the Carriage Return character
      rxBuff[rxCnt] = 0;                 // Terminate buffer string
      rxCnt = 0;                         // Reset data counter
      Received = 1;                      // Set Received data flag
    }
    else{
      rxBuff[rxCnt] = tmp;               // Populate buffer string with received data
      rxCnt = rxCnt + 1;                 // Increase data counter
    }
    U2RXIF_bit = 0;
}

// MCU initialization
void MCU_Init(){
  AD1PCFG     = 0xFFFF;              // Turning all analog pins to digital
  
  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320, 240);   // Initialize TFT display
  DrawFrame();

  Send_Button_Direction = 1;         // Set PORTD.B3 pin as input
  
  UART2_Init(19200);                 // Initialize UART module at 19200 bps
  Delay_ms(100);                     // Wait for UART module to stabilize
  
  U2IP0_bit = 0;                     // Set UART2 interrupt
  U2IP1_bit = 1;                     // Set interrupt priorities
  U2IP2_bit = 1;                     // Set UART2 interrupt to level 6

  U2RXIF_bit = 0;                    // Ensure interrupt not pending
  U2RXIE_bit = 1;                    // Enable intterupt
  EnableInterrupts();
}

void main() {
  MCU_Init();                            // Initialize the MCU
  while(1) {
    if(Button(&PORTD, 3, 1, 1)) {        // If PORTD.B3 is pressed :
      UART2_Write_Text("EchoTest\r");    // send text string
      Delay_ms(250);                     // wait for 250ms
    }
    
    if(Received){                        // If data is received via UART :
      updateLabel(rxBUFF, 150, 80, lab1);
      Received = 0;                      // clear received data flag
      Delay_ms(250);                     // wait for 250ms
    }
  }
}